<?php /**
 * The admin-specific functionality of the plugin.
 *
 * This class handles all the admin-related functionality for the Eimskip plugin.
 * It includes settings, menus, and other admin-specific features.
 *
 * @link       smartmedia.is
 * @since      1.0.0
 * @package    Tvgexpress
 * @subpackage Tvgexpress/admin
 */
class Tvg_Service {

	protected $_headers;
	protected $_apiUrl = 'https://appgw.eimskip.net/gateway/TVGXpressOperations/1.0/';
	protected $_response;
	protected $_client;
	protected $_service = 'TVGXpress';
	/**
	 * Construct function
	 *
	 * @since    1.0.0
	 * @param string $username Username.
	 * @param string $apiKey ApiKey to connect to service.
	 * @param bool   $test Test service or not.
	 */
	public function __construct( $username, $apiKey, $test = false ) {
		$this->_headers = array(
			'Content-Type'  => 'application/json',
			'Accept'        => 'application/json',
			'Authorization' => sprintf( 'Basic %s', base64_encode( $username . ':' . $apiKey ) ),
		);

		if ( $test ) {
			$this->_apiUrl = 'https://appgw-test.eimskip.net/gateway/TVGXpressOperations/1.0/';
		}

		$this->_client = new WP_Http();
	}



	/**
	 *
	 * This service is used to get labels for a given shipment. A label contains details about the shipment it
	 * refers to. A ZPL string or a PDF file can be requested.
	 *
	 * @param string $shipmentId - The shipment number for which to get labels.
	 * @param array  $params [labelType] - Enum: "ZPL_STRING", "PDF".
	 * @return array [status] - Segment that indicates whether the operation was successful or not.
	 */
	public function getLabels( $shipmentId, $params = array() ) {
		return $this->get( sprintf( 'labels/%s', $shipmentId ), $params );
	}

	/**
	 * Returns detailed information about the requested shipment.
	 *
	 * @param string $shipmentId ShipmentId to fetch.
	 * @return array [status] Segment that indicates whether the operation was successful or not.
	 */
	public function getShipment( $shipmentId ) {
		return $this->get( sprintf( 'shipments/info/%s', $shipmentId ) );
	}


	/**
	 * Returns detailed information about all shipments created within the specified time interval.
	 * Maximum date span is 30 days.
	 *
	 * @param date $dateFrom Pattern: yyyy-MM-dd HH:mm:ss.
	 * @param date $dateTo Pattern: yyyy-MM-dd HH:mm:ss.
	 *
	 * @return array [status] Segment that indicates whether the operation was successful or not.<br><br>
	 */
	public function getShipments( $dateFrom, $dateTo ) {
		return $this->get(
			'shipments/info',
			array(
				'dateFrom' => $dateFrom,
				'dateTo'   => $dateTo,
			)
		);
	}

	/**
	 * Returns key information about the requested shipment.
	 *
	 * @param string $shipmentId Reference number of the original shipment.
	 * @return array [status] Segment that indicates whether the operation was successful or not.
	 */
	public function getShipmentStatus( $shipmentId ) {
		return $this->get( sprintf( 'shipments/status/%s', $shipmentId ) );
	}
	/**
	 * Get Shipment status html
	 *
	 * @since    1.0.0
	 * @param string $shipmentId Shipment to fetch.
	 */
	public function getShipmentStatusHtml( $shipmentId ) {
		$response = $this->get( sprintf( 'shipments/status/%s', $shipmentId ) );

		$html = '';

		if ( ! empty( $response ) && '200' === $response->status->code ) {
			$statusIndex = 1;
			$statusIndex = ( strlen( $response->response->shipmentStatus->ReceivedDate ) > 0 ) ? 2 : $statusIndex;
			$statusIndex = ( strlen( $response->response->shipmentStatus->DeliveryRequestDate ) > 0 ) ? 3 : $statusIndex;
			$statusIndex = ( strlen( $response->response->shipmentStatus->DeliveryConfirmedDate ) > 0 ) ? 4 : $statusIndex;

			$html = '<ul class="tvg-shipping-status">
                <li class="' . esc_html( $this->getStatusClass( 1, $statusIndex ) ) . '">Pakki skráður</li>
                <li class="' . esc_html( $this->getStatusClass( 2, $statusIndex ) ) . '">Pakkinn kominn til flutningsaðila</li>
                <li class="' . esc_html( $this->getStatusClass( 3, $statusIndex ) ) . '">Pakkinn er á leiðinni til viðskiptavinar</li>
                <li class="' . esc_html( $this->getStatusClass( 4, $statusIndex ) ) . '">Pakki hefur komist til skila</li>
                </ul>';
		}
		return $html;
	}
	/**
	 * Get Status Class
	 *
	 * @since    1.0.0
	 * @param string $index Index to compare.
	 * @param string $statusIndex Status Index to verify.
	 */
	private function getStatusClass( $index, $statusIndex ): string {
		if ( $index === $statusIndex ) {
			return 'active';
		} elseif ( $index > $statusIndex ) {
			return 'disabled';
		} else {
			return 'done';
		}
	}
	/**
	 * This service is used to register a new shipment in the TVGXpress system. If registration is successful,
	 * the service returns a shipment number generated for the request.
	 *
	 * @param array $params [request].
	 * @return array [status] Segment that indicates whether the operation was.
	 */
	public function createShipment( $params ) {
		return $this->post( 'shipments', $params );
	}
	/**
	 * Get data
	 *
	 * @since    1.0.0
	 * @param string $url Url to get data.
	 * @param array  $params Params.
	 */
	protected function get( $url, $params = null ) {
		$data = array(
			'headers' => $this->_headers,
		);
		if ( ! empty( $params ) ) {
			$url = $url . '?' . http_build_query( $params );
		}

		try {
			$response    = wp_remote_get( $this->_apiUrl . $url, $data );
			$code        = wp_remote_retrieve_response_code( $response );
			$retResponse = json_decode( wp_remote_retrieve_body( $response ) );
			$this->log(
				wc_print_r(
					array(
						'code'     => $code,
						'url'      => $this->_apiUrl . $url,
						'data'     => $params,
						'response' => $retResponse,
					),
					true
				)
			);
			return $retResponse;
		} catch ( exception $e ) {
			$this->log(
				wc_print_r(
					array(
						'code'     => $e->getCode(),
						'url'      => $this->_apiUrl . $url,
						'data'     => $params,
						'response' => $e->getMessage(),
					),
					true
				)
			);
		}
	}
	/**
	 * Post data
	 *
	 * @since    1.0.0
	 * @param string $url Url to post data.
	 * @param array  $params Params to post.
	 */
	protected function post( $url, $params ) {

		$data = array(
			'headers'     => $this->_headers,
			'body'        => wp_json_encode( $params ),
			'data_format' => 'body',
		);

		try {
			$response    = wp_remote_post( $this->_apiUrl . $url, $data );
			$code        = wp_remote_retrieve_response_code( $response );
			$retResponse = json_decode( wp_remote_retrieve_body( $response ) );
			$this->log(
				wc_print_r(
					array(
						'code'    => $code,
						'url'     => $this->_apiUrl . $url,
						'data'    => $params,
						'rsponse' => $retResponse,
					),
					true
				)
			);
			return $retResponse;

		} catch ( exception $e ) {
			$this->log(
				wc_print_r(
					array(
						'code'    => $e->getCode(),
						'url'     => $this->_apiUrl . $url,
						'data'    => $params,
						'rsponse' => $e->getMessage(),
					),
					true
				)
			);
		}
	}
	/**
	 * Track package
	 *
	 * @since    1.0.0
	 * @param array $body body to track.
	 */
	public function track_package( $body = array() ) {

		$url = 'https://tvg-shopify-jacd3.ondigitalocean.app/api/collection/track';

		$args = array(
			'timeout' => 0,
			'method'  => 'POST',
			'headers' => array(
				'Content-Type' => 'application/json',
				'Connection'   => 'keep-alive',
			),
		);

		if ( ! empty( $body ) ) {
			$body         = wp_json_encode( $body );
			$args['body'] = $body;
		}
		try {
			$response    = wp_remote_request( $url, $args );
			$code        = wp_remote_retrieve_response_code( $response );
			$retResponse = json_decode( wp_remote_retrieve_body( $response ) );
			$this->log(
				wc_print_r(
					array(
						'code'     => $code,
						'url'      => $url,
						'data'     => json_decode( $body ),
						'response' => $retResponse,
					),
					true
				)
			);
			return $retResponse;
		} catch ( exception $e ) {
			$this->log(
				wc_print_r(
					array(
						'code'     => $e->getCode(),
						'url'      => $url,
						'data'     => json_decode( $body ),
						'response' => $e->getMessage(),
					),
					true
				)
			);
		}
	}
	/**
	 * Retrieve last response
	 *
	 * @return GuzzleHttp\Psr7\Response
	 */
	public function getResponse() {
		return $this->_response;
	}
	/**
	 * Log to log errors
	 *
	 * @since    1.0.0
	 * @param string $message Log message.
	 */
	protected function log( $message ) {
		if ( empty( $this->log ) ) {
			$this->log = wc_get_logger();
		}

		$this->log->debug( $message, array( 'source' => 'tvg-xpress' ) );

	}
	public function allowedhtml() {
		return array(
			'ul' => array(
				'class' => array(),
			),
			'li' => array(
				'class' => array(),
			),
		);
	}
}

